# coding=utf-8
import click
from mali_commands.config import Config


@click.group('auth')
def auth_commands():
    pass


@auth_commands.command('init')
@click.pass_context
@click.option('--webserver/--disable-webserver', default=True, required=False)
def init_auth(ctx, webserver):
    from .commons import pixy_flow

    ctx.obj.local_web_server = webserver

    access_token, refresh_token, id_token = pixy_flow(ctx.obj)

    config = Config()

    config.update_and_save({
        'token': {
            'access_token': access_token,
            'refresh_token': refresh_token,
            'id_token': id_token,
        }
    })



# coding=utf-8


def get_unique_keys(d):
    total_keys = []
    for obj in d:
        for key in obj.keys():
            if key in total_keys:
                continue

            total_keys.append(key)

    return total_keys


def dict_to_csv(d, fields):
    total_keys = fields or sorted(get_unique_keys(d))

    yield total_keys

    def get_row():
        for key in total_keys:
            yield obj.get(key, '')

    if isinstance(d, dict):
        d = [d]

    for obj in d:
        yield list(get_row())



# coding=utf-8
import click

try:
    import ConfigParser as configparser
except ImportError:
    import configparser

missing_link_config = 'missinglink.cfg'


class Config(object):
    def __init__(self):
        parser = configparser.RawConfigParser()
        parser.read([missing_link_config])

        self.parser = parser

    @property
    def token_config(self):
        return self.items('token')

    @property
    def refresh_token(self):
        return self.token_config.get('refresh_token')

    @property
    def id_token(self):
        return self.token_config.get('id_token')

    def items(self, section=None):
        try:
            return dict(self.parser.items(section))
        except configparser.NoSectionError:
            return {}

    def set(self, section, key, val):
        try:
            self.parser.add_section(section)
        except configparser.DuplicateSectionError:
            pass

        self.parser.set(section, key, val)

    def write(self, fo):
        self.parser.write(fo)

    def update_and_save(self, d):
        for section, section_values in d.items():
            for key, val in section_values.items():
                self.set(section, key, val)

        with open(missing_link_config, 'w') as configfile:
            self.write(configfile)




# coding=utf-8
import click
import requests

from mali_commands.tables import dict_to_csv
from .commons import add_to_data_if_not_none, create_validate_length
from terminaltables import PorcelainTable


@click.group('projects')
def projects_commands():
    pass


@projects_commands.command('list')
# deprecated but some clients use it
@click.option('--outputFormat', '-o', type=click.Choice(['tables', 'json']), default='tables', required=False)
@click.pass_context
def list_projects(ctx, outputformat):
    from .commons import handle_api

    result = handle_api(ctx.obj, requests.get, 'projects')

    json_format = ctx.obj.output_format == 'json' or outputformat == 'json'
    format_tables = not json_format

    if result is not None:
        if format_tables:
            fields = ['project_id', 'display_name', 'description', 'token', 'org']
            table_data = list(dict_to_csv(result.get('projects', []), fields))

            click.echo(PorcelainTable(table_data).table)
        else:
            click.echo(result)


max_project_display_name = 140
min_project_display_name = 1

max_project_description = 140
min_project_description = 0


@projects_commands.command('create')
@click.option(
    '--displayName', required=True, callback=create_validate_length(min_project_display_name, max_project_display_name))
@click.option(
    '--description', required=False, callback=create_validate_length(min_project_description, max_project_description))
@click.option('--org', required=False)
# deprecated but some clients use it
@click.option('--outputFormat', '-o', type=click.Choice(['tables', 'json']), default='tables', required=False)
@click.pass_context
def create_project(ctx, displayname, description, org, outputformat):
    from .commons import handle_api

    data = {}

    add_to_data_if_not_none(data, displayname, "display_name")
    add_to_data_if_not_none(data, org, "org")
    add_to_data_if_not_none(data, description, "description")

    result = handle_api(ctx.obj, requests.post, 'projects', data)

    json_format = ctx.obj.output_format == 'json' or outputformat == 'json'
    format_tables = not json_format

    if result is not None:
        if format_tables:
            fields = ['id', 'token']
            table_data = list(dict_to_csv(result, fields))

            click.echo(PorcelainTable(table_data).table)
        else:
            click.echo(result)


@projects_commands.command('transfer')
@click.option('--projectId', required=True)
@click.option('--transferTo', required=False)
@click.pass_context
def transfer_project(ctx, projectid, transferto):
    from .commons import handle_api

    data = {}

    add_to_data_if_not_none(data, transferto, "transfer_to")

    result = handle_api(
        ctx.obj, requests.post, 'projects/{project_id}/transfer'.format(project_id=projectid), data)

    format_tables = ctx.obj.output_format == 'tables'

    if result is not None:
        if format_tables:
            fields = ['ok']
            table_data = list(dict_to_csv(result, fields))

            click.echo(PorcelainTable(table_data).table)
        else:
            click.echo(result)



# coding=utf-8
import click
from mali_commands import auth_commands, projects_commands, orgs_commands
from mali_commands.config import Config


class Expando(object):
    pass

@click.group()
@click.option('--apiHost', default='https://missinglinkai.appspot.com', required=False)
@click.option('--host', default='https://missinglink.ai', required=False)
@click.option('--clientId', default='nbkyPAMoxj5tNzpP07vyrrsVZnhKYhMj', required=False)
@click.option('--auth0', default='missinglink', required=False)
@click.option('--outputFormat', '-o', type=click.Choice(['tables', 'json']), default='tables', required=False)
@click.pass_context
def cli(ctx, apihost, host, clientid, auth0, outputformat):
    ctx.obj = Expando()

    ctx.obj.api_host = apihost
    ctx.obj.host = host
    ctx.obj.client_id = clientid
    ctx.obj.auth0 = auth0
    ctx.obj.output_format = outputformat

    config = Config()
    ctx.obj.refresh_token = config.refresh_token
    ctx.obj.id_token = config.id_token

cli.add_command(auth_commands)
cli.add_command(projects_commands)
cli.add_command(orgs_commands)

if __name__ == "__main__":
    cli()



# coding=utf-8
import fudge
import httpretty

from mali import cli
from .base import BaseTest
from click.testing import CliRunner


class TestAuth(BaseTest):
    @httpretty.activate
    @fudge.patch('mali_commands.commons.pixy_flow')
    @fudge.patch('mali_commands.config.Config.update_and_save')
    def test_init_auth(self, pixy_flow, update_and_save_config):
        access_token = 'access_token'
        refresh_token = 'refresh_token'
        id_token = 'id_token'

        pixy_flow.expects_call().returns((access_token, refresh_token, id_token)).times_called(1)
        update_and_save_config.expects_call().with_args(
            {
                'token': {
                    'access_token': access_token,
                    'id_token': id_token,
                    'refresh_token': refresh_token
                }
            }
        ).returns(None).times_called(1)

        runner = CliRunner()
        result = runner.invoke(cli, ["auth", "init"], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)



# coding=utf-8
import fudge
import httpretty
from fudge.inspector import arg

from mali import cli
from .base import BaseTest
from click.testing import CliRunner


class TestOrgs(BaseTest):
    user_org = "user_org_" + BaseTest.some_random_shit()
    email_domain = "domain_%s@%s" % (BaseTest.some_random_shit(), BaseTest.some_random_shit())

    @httpretty.activate
    @fudge.patch('mali_commands.commons.handle_api')
    def test_list_orgs(self, handle_api):
        handle_api.expects_call().with_matching_args(
            arg.any(),  # ctx
            arg.any(),  # http method
            'orgs',
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ["orgs", "list"], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)

    @httpretty.activate
    @fudge.patch('mali_commands.commons.handle_api')
    def test_create_org(self, handle_api):
        handle_api.expects_call().with_matching_args(
            arg.any(),  # ctx
            arg.any(),  # http method
            'orgs',
            {
                'org': self.user_org,
            }
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ["orgs", "create", "--org", self.user_org], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)

    @httpretty.activate
    @fudge.patch('mali_commands.commons.handle_api')
    def test_auto_join_org_create(self, handle_api):
        handle_api.expects_call().with_matching_args(
            arg.any(),  # ctx
            arg.any(),  # http method
            'orgs/{org}/autoJoin'.format(org=self.user_org),
            {
                'domains': [self.email_domain],
            }
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ["orgs", "autoJoinDomain", "--org", self.user_org, "--domain", self.email_domain], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)



# coding=utf-8
import click
import requests

from mali_commands.tables import dict_to_csv
from .commons import add_to_data_if_not_none
from terminaltables import PorcelainTable


@click.group('orgs')
def orgs_commands():
    pass


@orgs_commands.command('list')
@click.pass_context
def list_orgs(ctx):
    from .commons import handle_api

    result = handle_api(ctx.obj, requests.get, 'orgs')

    format_tables = ctx.obj.output_format == 'tables'

    if result is not None:
        if format_tables:
            fields = ['org', 'display_name']
            table_data = list(dict_to_csv(result.get('orgs', []), fields))

            click.echo(PorcelainTable(table_data).table)
        else:
            click.echo(result)


@orgs_commands.command('create')
@click.option('--org', required=True)
@click.option('--displayName', required=False)
@click.option('--description', required=False)
@click.pass_context
def create_org(ctx, org, displayname, description):
    from .commons import handle_api

    data = {}

    add_to_data_if_not_none(data,  displayname, 'display_name')
    add_to_data_if_not_none(data, description, 'description')
    add_to_data_if_not_none(data, org, 'org')

    result = handle_api(ctx.obj, requests.post, 'orgs', data)

    format_tables = ctx.obj.output_format == 'tables'

    if result is not None:
        if format_tables:
            fields = ['ok']
            table_data = list(dict_to_csv(result, fields))

            click.echo(PorcelainTable(table_data).table)
        else:
            click.echo(result)


@orgs_commands.command('autoJoinDomain')
@click.option('--org', required=True)
@click.option('--domain', required=True, multiple=True)
@click.pass_context
def auto_join_domain(ctx, org, domain):
    from .commons import handle_api

    data = {}

    add_to_data_if_not_none(data, list(domain), 'domains')

    result = handle_api(ctx.obj, requests.post, 'orgs/{org}/autoJoin'.format(org=org), data)

    format_tables = ctx.obj.output_format == 'tables'

    if result is not None:
        if format_tables:
            fields = ['ok']
            table_data = list(dict_to_csv(result, fields))

            click.echo(PorcelainTable(table_data).table)
        else:
            click.echo(result)



# coding=utf-8
import fudge
import httpretty
from click.testing import CliRunner
from fudge.inspector import arg

from mali import cli
from mali_commands.projects import max_project_display_name, max_project_description, min_project_display_name
from tests.base import BaseTest


class TestProject(BaseTest):
    project_id = BaseTest.some_random_shit_number_int63()
    user_org = 'user_org_' + BaseTest.some_random_shit()

    @httpretty.activate
    def test_create_project_too_long_display_name(self):
        project_display_name = '9' * (max_project_display_name + 1)

        runner = CliRunner()
        params = ['projects', 'create', '--displayName', project_display_name]
        result = runner.invoke(cli, params, catch_exceptions=False)
        self.assertEquals(result.output, 'Usage: cli projects create [OPTIONS]\n\nError: Invalid value for "--displayName": needs to be shorter than %s characters\n' % max_project_display_name)

    @httpretty.activate
    def test_create_project_too_short_display_name(self):
        project_display_name = ''

        runner = CliRunner()
        params = ['projects', 'create', '--displayName', project_display_name]
        result = runner.invoke(cli, params, catch_exceptions=False)
        self.assertEquals(result.output, 'Usage: cli projects create [OPTIONS]\n\nError: Invalid value for "--displayName": needs to be longer than %s characters\n' % min_project_display_name)

    @httpretty.activate
    def test_create_project_too_long_description(self):
        project_description = '9' * (max_project_description + 1)
        project_display_name = 'display_' + self.some_random_shit()

        runner = CliRunner()
        params = ['projects', 'create', '--displayName', project_display_name, '--description', project_description]
        result = runner.invoke(cli, params, catch_exceptions=False)
        self.assertEquals(result.output, 'Usage: cli projects create [OPTIONS]\n\nError: Invalid value for "--description": needs to be shorter than %s characters\n' % max_project_description)

    @httpretty.activate
    @fudge.patch('mali_commands.commons.handle_api')
    def test_create_project_no_org(self, handle_api):
        project_display_name = 'display_' + self.some_random_shit()

        handle_api.expects_call().with_matching_args(
            arg.any(),  # ctx
            arg.any(),  # http method
            'projects',
            {
                'display_name': project_display_name
            },
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ['projects', 'create', '--displayName', project_display_name], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)

    @httpretty.activate
    @fudge.patch('mali_commands.commons.handle_api')
    def test_create_project_with_org(self, handle_api):
        org = 'org_' + self.some_random_shit()
        project_display_name = 'display_' + self.some_random_shit()

        handle_api.expects_call().with_matching_args(
            arg.any(),  # ctx
            arg.any(),  # http method
            'projects',
            {
                'display_name': project_display_name,
                'org': org,
            },
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ['projects', 'create', '--displayName', project_display_name, '--org', org], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)

    @httpretty.activate
    @fudge.patch('mali_commands.commons.handle_api')
    def test_list_projects(self, handle_api):
        handle_api.expects_call().with_matching_args(
            arg.any(),  # ctx
            arg.any(),  # http method
            'projects',
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ["projects", "list"], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)

    @httpretty.activate
    @fudge.patch('mali_commands.commons.handle_api')
    def test_list_projects_params_json(self, handle_api):
        handle_api.expects_call().with_matching_args(
            arg.any(),  # ctx
            arg.any(),  # http method
            'projects',
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ["-o", "json", "projects", "list"], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)

    @httpretty.activate
    @fudge.patch('mali_commands.commons.handle_api')
    def test_list_projects_params_json_old(self, handle_api):
        handle_api.expects_call().with_matching_args(
            arg.any(),  # ctx
            arg.any(),  # http method
            'projects',
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ["projects", "list", "-o", "json"], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)

    @httpretty.activate
    @fudge.patch('mali_commands.commons.handle_api')
    def test_transfer_project(self, handle_api):
        handle_api.expects_call().with_matching_args(
            arg.any(),  # ctx
            arg.any(),  # http method
            'projects/{project_id}/transfer'.format(project_id=self.project_id),
            {
                'transfer_to': self.user_org,
            }
        ).returns({})

        runner = CliRunner()
        result = runner.invoke(cli, ["projects", "transfer", "--projectId", self.project_id, "--transferTo", self.user_org], catch_exceptions=False)
        self.assertEquals(result.exit_code, 0)



# coding=utf-8
import os
from setuptools import setup, find_packages

build = os.environ['PIP_BUILD']
keywords = os.environ['PIP_KEYWORDS']


version = '0.{}'.format(build)

setup(
    name='mali',
    version=version,
    py_modules=['mali'],
    packages=find_packages(),
    install_requires=[
        'click',
        'requests',
        'terminaltables',
        'pip',
    ],
    entry_points='''
        [console_scripts]
        mali=mali:cli
    ''',
)
